@@ -39,6 +39,7 @@
 #include <sys/resource.h>
 #include <sys/wait.h>
 #include <pthread.h>
+#include <sys/cygwin.h>
 #endif
 #include "win.h"
 
@@ -61,9 +62,6 @@ extern int parse_file(FILE * fp);
 /* Currently in use command ID, incremented each new menu item created */
 static int g_cmdid = STARTMENUID;
 
-/* Local function to handle comma-ified icon names */
-static HICON LoadImageComma(char *fname, int sx, int sy, int flags);
-
 /*
  * Creates or appends a menu from a MENUPARSED structure
  */
@@ -208,18 +206,21 @@ ReloadEnumWindowsProc(HWND hwnd, LPARAM lParam)
  * Set custom icons and menus again.
  */
 static void
-ReloadPrefs(void)
+ReloadPrefs(winPrivScreenPtr pScreenPriv)
 {
     int i;
 
 #ifdef XWIN_MULTIWINDOW
+    winScreenInfo *pScreenInfo = pScreenPriv->pScreenInfo;
+
     /* First, iterate over all windows, deleting their icons and custom menus.
      * This is really only needed because winDestroyIcon() will try to
      * destroy the old global icons, which will have changed.
      * It is probably better to set a windows USER_DATA to flag locally defined
      * icons, and use that to accurately know when to destroy old icons.
      */
-    EnumThreadWindows(g_dwCurrentThreadID, ReloadEnumWindowsProc, FALSE);
+    if (pScreenInfo->fMultiWindow)
+        EnumThreadWindows(g_dwCurrentThreadID, ReloadEnumWindowsProc, FALSE);
 #endif
 
     /* Now, free/clear all info from our prefs structure */
@@ -264,12 +265,12 @@ ReloadPrefs(void)
     g_hSmallIconX = NULL;
 
 #ifdef XWIN_MULTIWINDOW
-    winInitGlobalIcons();
-#endif
+    if (pScreenInfo->fMultiWindow) {
+        winInitGlobalIcons();
 
-#ifdef XWIN_MULTIWINDOW
-    /* Rebuild the icons and menus */
-    EnumThreadWindows(g_dwCurrentThreadID, ReloadEnumWindowsProc, TRUE);
+        /* Rebuild the icons and menus */
+        EnumThreadWindows(g_dwCurrentThreadID, ReloadEnumWindowsProc, TRUE);
+    }
 #endif
 
     /* Whew, done */
@@ -409,7 +410,7 @@ ExecAndLogThread(void *cmd)
  * Return TRUE if command is proccessed, FALSE otherwise.
  */
 Bool
-HandleCustomWM_COMMAND(HWND hwnd, int command)
+HandleCustomWM_COMMAND(HWND hwnd, WORD command, winPrivScreenPtr pScreenPriv)
 {
     int i, j;
     MENUPARSED *m;
@@ -482,13 +483,17 @@ HandleCustomWM_COMMAND(HWND hwnd, int command)
                                      HWND_TOPMOST,
                                      0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
 #if XWIN_MULTIWINDOW
-                    /* Reflect the changed Z order */
-                    winReorderWindowsMultiWindow();
+                    {
+                        winScreenInfo *pScreenInfo = pScreenPriv->pScreenInfo;
+                        if (pScreenInfo->fMultiWindow)
+                            /* Reflect the changed Z order */
+                            winReorderWindowsMultiWindow();
+                    }
 #endif
                     return TRUE;
 
                 case CMD_RELOAD:
-                    ReloadPrefs();
+                    ReloadPrefs(pScreenPriv);
                     return TRUE;
 
                 default:
@@ -519,8 +524,10 @@ SetupSysMenu(HWND hwnd)
     pWin = GetProp(hwnd, WIN_WINDOW_PROP);
 
     sys = GetSystemMenu(hwnd, FALSE);
-    if (!sys)
+    if (!sys) {
+        ErrorF("SetupSysMenu: GetSystemMenu() failed for HWND %p\n", hwnd);
         return;
+    }
 
     if (pWin) {
         /* First see if there's a class match... */
@@ -576,7 +583,7 @@ winOverrideDefaultIcon(int size)
     HICON hicon;
 
     if (pref.defaultIconName[0]) {
-        hicon = LoadImageComma(pref.defaultIconName, size, size, 0);
+        hicon = LoadImageComma(pref.defaultIconName, pref.iconDirectory, size, size, 0);
         if (hicon == NULL)
             ErrorF("winOverrideDefaultIcon: LoadImageComma(%s) failed\n",
                    pref.defaultIconName);
@@ -598,9 +605,12 @@ winTaskbarIcon(void)
     hicon = 0;
     /* First try and load an overridden, if success then return it */
     if (pref.trayIconName[0]) {
-        hicon = LoadImageComma(pref.trayIconName,
+        hicon = LoadImageComma(pref.trayIconName, pref.iconDirectory,
                                GetSystemMetrics(SM_CXSMICON),
                                GetSystemMetrics(SM_CYSMICON), 0);
+        if (hicon == NULL)
+            ErrorF("winTaskbarIcon: LoadImageComma(%s) failed\n",
+                   pref.trayIconName);
     }
 
     /* Otherwise return the default */
@@ -615,17 +625,18 @@ winTaskbarIcon(void)
 }
 
 /*
+ * Handle comma-ified icon names
+ *
  * Parse a filename to extract an icon:
  *  If fname is exactly ",nnn" then extract icon from our resource
  *  else if it is "file,nnn" then extract icon nnn from that file
  *  else try to load it as an .ico file and if that fails return NULL
  */
-static HICON
-LoadImageComma(char *fname, int sx, int sy, int flags)
+HICON
+LoadImageComma(char *fname, char *iconDirectory, int sx, int sy, int flags)
 {
     HICON hicon;
     int i;
-    char file[PATH_MAX + NAME_MAX + 2];
 
     /* Some input error checking */
     if (!fname || !fname[0])
@@ -641,31 +652,67 @@ LoadImageComma(char *fname, int sx, int sy, int flags)
                           MAKEINTRESOURCE(i), IMAGE_ICON, sx, sy, flags);
     }
     else {
+        char *file = malloc(PATH_MAX + NAME_MAX + 2);
+        Bool convert = FALSE;
+
+        if (!file)
+            return NULL;
+
         file[0] = 0;
-        /* Prepend path if not given a "X:\" filename */
+
+        /* If fname starts 'X:\', it's an absolute Windows path, do nothing */
         if (!(fname[0] && fname[1] == ':' && fname[2] == '\\')) {
-            strcpy(file, pref.iconDirectory);
-            if (pref.iconDirectory[0])
-                if (fname[strlen(fname) - 1] != '\\')
-                    strcat(file, "\\");
+#ifdef  __CYGWIN__
+            /* If fname starts with '/', it's an absolute cygwin path, we'll
+               need to convert it */
+            if (fname[0] == '/') {
+                convert = TRUE;
+            }
+            else
+#endif
+            if (iconDirectory) {
+                /* Otherwise, prepend the default icon directory, which
+                   currently must be in absolute Windows path form */
+                strcpy(file, iconDirectory);
+                if (iconDirectory[0])
+                    if (iconDirectory[strlen(iconDirectory) - 1] != '\\')
+                        strcat(file, "\\");
+            }
         }
         strcat(file, fname);
 
+        /* Trim off any ',index' */
         if (strrchr(file, ',')) {
-            /* Specified as <fname>,<index> */
-
             *(strrchr(file, ',')) = 0;  /* End string at comma */
             i = atoi(strrchr(fname, ',') + 1);
+        }
+        else {
+            i = -1;
+        }
+
+#ifdef  __CYGWIN__
+        /* Convert from Cygwin path to Windows path */
+        if (convert) {
+            char *converted_file = cygwin_create_path(CCP_POSIX_TO_WIN_A | CCP_ABSOLUTE, file);
+            if (converted_file) {
+                free(file);
+                file = converted_file;
+            }
+        }
+#endif
+
+        if (i >= 0) {
+            /* Specified as <fname>,<index> */
             hicon = ExtractIcon(g_hInstance, file, i);
         }
         else {
-            /* Just an .ico file... */
-
+            /* Specified as just an .ico file */
             hicon = (HICON) LoadImage(NULL,
                                       file,
                                       IMAGE_ICON,
                                       sx, sy, LR_LOADFROMFILE | flags);
         }
+        free(file);
     }
     return hicon;
 }
@@ -687,7 +734,7 @@ winOverrideIcon(char *res_name, char *res_class, char *wmName)
             if (pref.icon[i].hicon)
                 return pref.icon[i].hicon;
 
-            hicon = LoadImageComma(pref.icon[i].iconFile, 0, 0, LR_DEFAULTSIZE);
+            hicon = LoadImageComma(pref.icon[i].iconFile, pref.iconDirectory, 0, 0, LR_DEFAULTSIZE);
             if (hicon == NULL)
                 ErrorF("winOverrideIcon: LoadImageComma(%s) failed\n",
                        pref.icon[i].iconFile);
